//+------------------------------------------------------------------+
//|                          BugNoFileWritten_agentInsteadCommon.mq5 |
//|                                    Copyright 2023, Julian Jaeger |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, Julian Jaeger"
#property link      "https://www.mql5.com"
#property version   "1.00"

#include <MT4Orders.mqh>

MqlTick mqltick;
int counter = 0,slippage = 0, m_filehandle;
double AccountBalance,lotStep,_Pips,PipValue,_price,_SL,_TP,ticksize,tickvalue, Ask, _lotsize,vol, _vol;
double atr_array[];
int AtrHandle;

//+------------------------------------------------------------------+
input double RiskPercent = 1;
input double Multiplier = 1.0;//TPSL_multiplier
input string filename = "DemoFile";
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   PipValue = c_PipValue(_Pips);
   lotStep=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_STEP);
   AtrHandle=iATR(_Symbol,PERIOD_D1,7);
   ArraySetAsSeries(atr_array,true);

//---
   return(INIT_SUCCEEDED);
  }

//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
    if(isNewBar() && !IsOpen() && CheckIfMarketOpen())// && SymbolIsSynchronized(_Symbol))
      {
         SymbolInfoTick(_Symbol,mqltick);
         Ask = mqltick.ask;
         if(!TP_SL_LOT(vol,_SL,_TP))
           {
             Print("TP_SL_LOT false");
             return;
           }
         if(!CheckVolumeValue(vol,_vol))
           {
             Print("CheckVolumeValue false");
             return;
           }
         if(!EnoughMargin(_Symbol,_vol,OP_BUY, _price))
           {
             Print("EnoughMargin false");
             return;
           }
         OrderSend(_Symbol,OP_BUY,_vol,_price,slippage,_SL,_TP,"TestTrade",1);
      }
  }
//+------------------------------------------------------------------+
//| OnTester
//+------------------------------------------------------------------+
double OnTester(void)
 {
   double ret=0.0;
   short Rrow[];
   string row;
   ret = TesterStatistics(STAT_TRADES);
   
//--frames
   row = DoubleToString(ret);
   StringToShortArray(row,Rrow);
   string fram_name = "backtest" ;
   double frame_val = 0; 
   if(MQLInfoInteger(MQL_FORWARD) == 1)
	{
	 fram_name = "forward";
	 frame_val = 1;
	}
  if(!FrameAdd(fram_name,1,frame_val,Rrow))
  {
    printf("FrameAdd failed with error %i",GetLastError());
  }
  return(ret);
 }
//+------------------------------------------------------------------+
//| TesterDeinit function                                            |
//+------------------------------------------------------------------+
void OnTesterDeinit()
  {
//---
	   FileDelete(filename+".csv");
	   
	   FrameFirst();
	   ulong pass;
	   string name;
	   long id;
	   double value;
	   ushort data[];
	   
	   if(!FrameNext(pass,name,id,value,data))
		 {
		   printf("error #%i with FrameNext",GetLastError());
		 }
	   string receivedData=ShortArrayToString(data);
	   string datalog = DoubleToString(value,0)+"-";
	   StringAdd(datalog,receivedData);

	   m_filehandle=FileOpen(filename+".csv" ,FILE_READ|FILE_WRITE|FILE_CSV);
	   if(m_filehandle==INVALID_HANDLE) 
		{
		 printf("Error %i creating tester file",GetLastError());
		}
		else
		  {
			FileSeek(m_filehandle,0,SEEK_END);
			FileWrite(m_filehandle,datalog);
		  } 

	   while(FrameNext(pass,name,id,value,data))
		  {
			receivedData=ShortArrayToString(data);
			datalog = DoubleToString(value,0)+"-";
			StringAdd(datalog, receivedData);
			FileSeek(m_filehandle,0,SEEK_END);
			FileWrite(m_filehandle,datalog);
		  }  

	  FileClose(m_filehandle);
	}   
//+------------------------------------------------------------------+
bool CheckVolumeValue(double volume, double &c_volume)
  {
//--- minimal lotsize
   double min_volume=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MIN);
   c_volume = volume;
   if(volume<min_volume)
     {
      c_volume=min_volume;
     }

//--- maximal lotsize
   double max_volume=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MAX);
   if(volume>max_volume)
     {
      c_volume=max_volume;
     }

//--- step
   double volume_step=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_STEP);

   int ratio=(int)MathRound(volume/volume_step);
   if(MathAbs(ratio*volume_step-volume)>0.0000001)
     {
      PrintFormat("Volume is not a multiple of the minimal step SYMBOL_VOLUME_STEP=%.2f, the closest correct volume is %.2f",
                               volume_step,ratio*volume_step);
      return(false);
     }
   return(true);
  }
//+------------------------------------------------------------------+
bool EnoughMargin(string symb,double lots,ENUM_ORDER_TYPE type, double &price)
{
   if(!AccBalance(AccountBalance))
    {
      return(false);
    }
   SymbolInfoTick(symb,mqltick);
   price=mqltick.ask;
   if(type==ORDER_TYPE_SELL)
      {
         price=mqltick.bid;
      }
   if(price == 0)
     {
       return(false);
     }
   double margin,free_margin=AccountInfoDouble(ACCOUNT_MARGIN_FREE);
   if(!OrderCalcMargin(type,symb,lots,price,margin))
      {
         Alert("Error in ",__FUNCTION__," code=",GetLastError());
         return(false);
      }
   if(margin>free_margin)
      {
         Alert("Not enough money (max margin reached) for ",EnumToString(type)," ",lots," ",symb," Error code=",GetLastError());
         return(false);
      }
   return(true);
}
//+------------------------------------------------------------------+
	bool AccBalance(double balance)
	{
	balance = AccountInfoDouble(ACCOUNT_BALANCE);
	if(balance > 0)
	  {
	    return(true);
	  }
	else
	  {
	    return(false);
	  }
	}
//+------------------------------------------------------------------+
bool isNewBar()
	  {
	   static datetime last_time=0;
	   datetime lastbar_time=(datetime)SeriesInfoInteger(_Symbol,PERIOD_H1,SERIES_LASTBAR_DATE);
	   if(last_time==0)
		 {
		  last_time=lastbar_time;
		  return(false);
		 }
	   if(last_time!=lastbar_time)
		 {
		  last_time=lastbar_time;
		  return(true);
		 }
	   return(false);
	  }
//+------------------------------------------------------------------+
bool IsOpen()
   {
      if(!OrderSelect(OrdersTotal()-1,SELECT_BY_POS))
   	 {
   	   return(false);
   	 }
      return(true);  
   }
//+------------------------------------------------------------------+
double c_PipValue(double &c_Pips)
{  	
      ticksize=SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_SIZE);
	   tickvalue=SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_VALUE);
	   if (_Digits/2 == 1){c_Pips=_Point;}
	   else if (_Digits / 2 != 1){c_Pips=_Point*10;}
	   double adjContractSize=tickvalue/ticksize;
	   return(PipValue = adjContractSize * c_Pips);
}
//+------------------------------------------------------------------+
bool CheckIfMarketOpen()
  {
	  if(!TerminalInfoInteger(TERMINAL_TRADE_ALLOWED) && !MQLInfoInteger(MQL_TESTER))
		   {
			 return(false);
		   }
	  return(true);  
  }
//+------------------------------------------------------------------+
bool TP_SL_LOT(double &LotSize, double &StopLoss, double &TakeProfit)
{
   if(!AccBalance(AccountBalance))
   {
     return(false);
   }
   //ATR
      CopyBuffer(AtrHandle,0,0,3,atr_array);
      double ATR=atr_array[1];   
      
   //TP  
      double TakeProfitPips = ATR * Multiplier / _Pips;	   
   //SL   
      double StopLossPips = ATR * Multiplier / _Pips;
   //lotsize
   if(StopLossPips>0 && PipValue>0 && lotStep>0)
      {
      	double Riskvalue = AccountBalance * (RiskPercent / 100);
      	double CalclotSize = Riskvalue / StopLossPips / PipValue; //calculate lotSize
      	double CalclotSizeFloor=MathFloor(CalclotSize/lotStep)*lotStep;
      	LotSize = NormalizeDouble(CalclotSizeFloor,2);
      }
      StopLoss = MathFloor((Ask-(StopLossPips*_Pips))/ticksize)*ticksize;
      TakeProfit = MathFloor((Ask+(TakeProfitPips*_Pips))/ticksize)*ticksize;
   
   return(true);
 }
	 